#!/usr/bin/perl -w
#############################################################################
# osxcelot.cgi - this is the generic wrapper around our plugins.            #
# last tweaked by morbus iff <morbus@disobey.com>, 2002-12-06.              #
# see also: http://www.disobey.com/osxcelot/ for more information.          #
#############################################################################
use strict; $|++;                                  # strict and flush.      #
use FindBin qw($Bin);                              # where we are, HmMmM?   #
use File::Spec::Functions;                         # path handling love.    #
BEGIN { unshift(@INC, catdir($Bin, "lib")); }      # add to our lookups.    #
                                                   #                        #
use CGI qw/:standard/;                             # parameters, webbased.  #
use Getopt::Long;                                  # parameters, shell.     #
use Text::Template;                                # templating lovejoy.    #
Text::Template->always_prepend("use strict;");     # use the force, dammit! #
#############################################################################

# determine our plugin location and taint.
my $plugin_dir = catdir($Bin, "plugins");
if ($plugin_dir =~ /^([\w.\(\)\/ ~-]+)$/) { $plugin_dir = $1; }
else { die "Taint problem"; } # we need a better error. hoohah.

# and our main template full path. we get our output 
# format from a passed parameter, but we default to html.
my %options; GetOptions(\%options, 'plain');
my $template_file; # final path stored here.
if ($options{plain} or param('plain') or !$ENV{REMOTE_ADDR}) {
       $template_file = catdir($Bin, "templates", "plain.template"); }
else { $template_file = catdir($Bin, "templates", "html.template"); }

# send our content-type if we're in a browser.
if (not $options{plain} and param('plain')) { print "Content-type: text/plain\n\n"; }
elsif (not $options{plain} and not param('plain') and $ENV{REMOTE_ADDR}) { print "Content-type: text/html\n\n"; }

# and send our final template results (either html or plain).
my $template = Text::Template->new( SOURCE => $template_file, DELIMITERS => [ '[$', '$]' ] )
                   or die "The template could not be loaded, which is rather unfortunate";
print $template->fill_in(); # that's it, we're done in here. whoo.

#############################################################################
# subroutines. they do most of the work. called from the T::T template.     #
#############################################################################

# return plugin html.
sub get_plugin_html {
   my ($plugin_path) = @_; # full path to our plugin.
   open(PLUGIN, "\"$plugin_path\" -m |") or return 0;
   my @responses = <PLUGIN>; close(PLUGIN);
   return join("", @responses);
}

# return plugin plain.
sub get_plugin_plain {
   my ($plugin_path) = @_; # full path to our plugin.
   open(PLUGIN, "\"$plugin_path\" |") or return 0;
   my @responses = <PLUGIN>; close(PLUGIN);
   return join("", @responses);
}

# return small purpose blurb.
sub get_plugin_purpose {
   my ($plugin_path) = @_; # full path to our plugin.
   open(PLUGIN, "\"$plugin_path\" -p |") or return 0;
   my $response = <PLUGIN>; close(PLUGIN);
   chop($response); return $response;
}

# return title/version.
sub get_plugin_title {
   my ($plugin_path) = @_; # full path to our plugin.
   open(PLUGIN, "\"$plugin_path\" -v |") or return 0;
   my $response = <PLUGIN>; close(PLUGIN);
   chop($response); return $response;
}

# open our plugin directory
# and return a list of full-
# pathed plugins we've found.
sub return_plugin_paths {
   opendir(PLUGINS, $plugin_dir) or die $!;
   my @plugins = grep /\.plugin$/, readdir(PLUGINS);
   closedir(PLUGINS); @plugins = sort @plugins;
   foreach my $plugin (@plugins) {
      if ($plugin =~ /^([-\w.]+)$/) { $plugin = $1; }
      else { next; } # taint plugin names for safety.
      $plugin = catfile($plugin_dir, $plugin);
   } return @plugins;
}

exit;
